
;	DTMF-Geber mit I2C-Schnittstelle
;	Uwe Hennig, DL6INT
;
;	letzte nderung: 03.12.2000
;
;	Prinzip:
;	Beide gleichzeitig zu erzeugenden Frequenzen werden jeweils
;	durch Timer-Interrupts erzeugt. Die Timer werden im 16-Bit-Modus
;	betrieben und bei jedem Interrupt auf den erforderlichen Wert
;	eingestellt. In den Interruptroutinen wird jeweils ein open-drain-
;	Ausgang des Controllers umgeschaltet. Die Addition der
;	beiden Kanle erfolgt ber ein Widerstandsnetzwerk.
;
;	Der Code des zu erzeugenden Tones wird ber I2C empfangen.
;	Whrend der Tonausgabe sind zwei Enable-Ausgnge low.

;	*

; to do:
;
	
	cpu	8051
	include	stddef51.inc	

;--------------------------------------------------------------------------
;	Belegung der Ports:
;
SCL	equ	P1.7
SDA	equ	P1.6

EnabA	equ	P1.4
EnabB	equ	P1.3

out1	equ	P1.1
out0	equ	P1.0

LED	equ	P3.7
LED2	equ	P3.5

;--------------------------------------------------------------------------
;
;	Speicherbelegung
;
;	Registerbank 0:		0..7
; Register:
;	r0: indexregister
;	r1: 
;	r2,r3: Warteschleife, allgemeines
;	r4,r5,r6: Timerinterruptroutine
;	r7: 

set0L	equ	20
set0H	equ	21

set1L	equ	22
set1H	equ	23

zeichen	equ	24

sumL	equ	25
sumM	equ	26
sumH	equ	27
n	equ	28
bits0	equ	29
bits1	equ	30

BitReg0	equ	32

; Bitdefinitionen:

;********************************************************************
	org	0
	ajmp	reboot

;---------------------------------------------------------
	org	0bh	; Interruptadresse timer0
	cpl	out0
	call	reti
	sjmp	itimer0

;---------------------------------------------------------
	org	1bh	; Interruptadresse timer1
	cpl	out1
	call	reti
	push	acc
	push	psw
	clr	TR1	; 1 Timer stop
	mov	a,TL1   ; 1 Auslesen
	add	a,set1L ; 1 addieren
	mov	TL1,a   ; 1 rckschreiben
	mov	a,#0	; 1 (TH1 ist ohnehin 0)
	addc	a,set1H ; 1
	mov	TH1,a	; 1 TH1 setzen
	setb	TR1	; 1 Timer start
	; Zeit aufsummieren
	; xxxxx provisorisch: eins weiterzhlen xxxxx
	mov	a,sumL
	add	a,#1
	mov	sumL,a
	mov	a,sumM
	addc	a,#0
	mov	sumM,a
	pop	psw
	pop	acc
	ret
;---------------------------------------------------------------
itimer0:
	push	acc
	push	psw

	; Die Zeit vom Auslsen des Interrupts bis zur Ausfhrung
	; kann unterschiedlich sein [Roth, S.4-5], deshalb den Timer
	; auslesen und verrechnen.
	; Ein berlauf bei der Addition von TL0 und set0L kann je nach
	; einzustellendem Wert auftreten und wird bercksichtigt.
	; In set0H##set0L mu der um (7 oder 8?) erhhte Wert stehen, der
	; eigentlich zur Erzeugung der Frequenz notwendig wre, weil
	; die Zeit zwischen Anhalten und Neustart des Timers weiter
	; luft.
	; 
	clr	TR0	; 1 Timer stop
	mov	a,TL0   ; 1 Auslesen
	add	a,set0L ; 1 addieren
	mov	TL0,a   ; 1 rckschreiben
	mov	a,#0	; 1 (TH0 ist ohnehin 0)
	addc	a,set0H ; 1
	mov	TH0,a	; 1 TH0 setzen
	setb	TR0	; 1 Timer start
	pop	psw
	pop	acc
	ret

reti:	reti

;-----------------------------------------------------------
reboot:
	; Initialisierung:
	
	; Stackpointer setzen:
	mov	SP,#6fh

	; blinken:
	clr	LED
	call	waitmx
	setb	LED

	; Variablen:
	clr	A
	
	; zum Testen: 1750 Hz * 2 entspricht 357 Zyklen,
	; 65536-375=65179,
	;+7=65187=0xFEA2
	;mov	set0H,#0feh
	;mov	set0L,#0a2h
	;mov	set1H,#0fdh
	;mov	set1L,#096h

	; Timer init
	mov	TMOD,#00010001b	; beide Timer 16 Bit
	mov	TCON,#0 ; vorerst die Timer noch nicht laufen lassen
	mov	IE,#8ah	; Freigabe beide Timer-Int

	mov	zeichen,#0


idle:	; *********** Hauptschleife ********************************

z1: ; passiver Wartezustand
	jnb	SCL,z1	; warten auf Clock H
	jb	SDA,z1  ; warten auf Data L

z2:
	jb	SDA,z1 ; ungltig->passiv
	jb	SCL,z2 ; warten auf Clock L (2.Teil der Startbedingung)

; Startbedingung passiert
	; hier Bitzhler rcksetzen
	mov	n,#0
z3:
	mov	c,SDA  ;1  Data schon mal sampeln
	jnb	SCL,z3 ;2  warten auf positive Clock-Flanke
	; Data steht in carry
	jc	rec1 ;2
	; Null-Bit empfangen
	call	rcvBit ;2
z4:	
	jb	SDA,z1 ; 2 Data L->H bei Clock H ist Stopbedingung
	jnb	SCL,z3 ; 2 bei Clock L nchstes Bit erwarten
	sjmp	z4 ; 2 sonst warten

rec1:	; Eins-Bit empfangen
	call	rcvBit
z5:
	jnb	SDA,z2 ; Data H->L bei Clock H ist Startbedingung
	jnb	SCL,z3 ; bei Clock L nchstes Bit erwarten
	sjmp	z5 ; sonst warten
;---------------------------------------------------

rcvBit:
	mov	a,bits1 ; 1
	rlc	a ; 1
	mov	bits1,a ; 1
	mov	a,bits0 ; 1
	rlc	a ; 1
	mov	bits0,a ; 1
	inc	n ; 2
	mov	a,n ; 1
	cjne	a,#16,nextbit ; 2
	; 16 Bit empfangen, das sind
	;  7 Adrebit
	;  1 R/W
	;  1 Ack
	;  7 Daten
	; Das 8.Datenbit und das nachfolgende Ack fallen untern Tisch.
	cpl	LED
	mov	a,bits0
	cjne	a,#0aah,ichnicht
	cpl	LED2
	; ich bin gemeint
	mov	a,bits1
	rl	a
	swap	a
	mov	zeichen,a ; die oberen 4 Bit des Datenbytes sind der Zeichencode
	call	zei2frq
	call	waitEnd
	sjmp	ichnicht
nextbit:
	ret ; 2
ichnicht:
	pop	acc ; Rcksprungadresse ist Abfall
	pop	acc
	ajmp	z1


;****************************************************************
waitEnd:
	mov	a,sumM
	cjne	a,#1,we1
we1:	
	jc	waitEnd
	mov	TCON,#0 ; Ton aus
	setb	EnabA ; beide Enable-Ausgnge passiv
	setb	EnabB
	ret

;*****************************************************************
;
;   Zeichen zu Frequenzen
;
;   in: 4-Bit-Code in zeichen
;
zei2frq:
	; Ausgabe stoppen
	mov	TCON,#0

	; neue Timer-Einstellungen berechnen
	; timer 0
	mov	a,zeichen
	anl	a,#3	; untere 2 Bit bestimmen hohe Frequenz
	rl	a	; mal zwei
	mov	dptr,#frqTabH
	push	acc
	movc	a,@a+dptr
	mov	set0L,a
	pop	acc
	inc	a
	movc	a,@a+dptr
	mov	set0H,a

	; timer 1
	mov	a,zeichen
	anl	a,#0ch	; obere zwei Bit des 4-Bit-Codes bestimmen untere Freq
	rr	a	; durch zwei
	mov	dptr,#frqTabL
	push	acc
	movc	a,@a+dptr
	mov	set1L,a
	pop	acc
	inc	a
	movc	a,@a+dptr
	mov	set1H,a

	; Beide Timer so einstellen, da sie in Krze berlaufen,
	; jedoch (zumindest beim Start) nicht gleichzeitig
	mov	TH0,#255
	mov	TL0,#128
	mov	TH1,#255
	mov	TL1,#0

	; Zeit-Summe auf Null stellen
	clr	a
	mov	sumL,a
	mov	sumM,a
	mov	sumH,a
	
	; Enable-Ausgnge aktivieren
	clr	EnabA
	clr	EnabB

	; und beide Timer loslaufen lassen
	mov	TCON,#50h  

	ret	


frqTabL:
	; unteres Frequenzpaket
	; 697 Hz  -> +7=64646=0xFC86
	dw	0fc86h
	; 770 Hz  -> +7=64731=0xFCDB
	dw	0fcdbh
	; 852 Hz  -> +7=64809=0xFD29
	dw	0fd29h
	; 941 Hz  -> +7=64879=0xFD6F
	dw	0fd6fh
frqTabH:
	; oberes Frequenzpaket
	; 1209 Hz -> +7=65026=0xFE02
	dw	0fe02h
	; 1336 Hz -> +7=65075=0xFE33
	dw	0fe33h
	; 1477 Hz -> +7=65120=0xFE60
	dw	0fe60h
	; 1632 Hz -> +7=65160=0xfe88
	dw	0fe88h

;**********************************************************************
;	Warteschleife
;	(mit 3 Einsprungmarken)
;	VR: r2,r3 haben am Ende den Wert 0
wait5:	mov	r2,#12
	sjmp	wait
waitmx:	mov	r2,#0
wait:	mov	r3,#0
wt_1:	djnz	r3,wt_1
	djnz	r2,wait
	ret


copyri:
	db	"DTMF-I2C 20001203 by DL6INT"
