/*************************************************************************
 *
 * Siemens C5 Steuerung
 *
 * C-Teil des Programms
 *
 * (c) Thomas Mller, 2001
 *
 * $RCSfile: c5_c.c $
 * $Revision: 1.5 $
 * $Date: 2001/03/26 05:18:00 $
 *
 */

#include "c5_c.h"

#ifndef C5TEST

#pragma aux (pascal) default
#pragma aux default far

#endif

extern char stellen;
extern char crs_pos;
extern char crs_buf;
extern char hour;
extern char min;
extern char sec;
extern char day;
extern char mon;
extern char year;
extern char hsec;
extern char hsec_alt;
extern char buf1cc7h[];
extern char txbuffer[];
extern char empfbuffer1[];
extern char telegrnummer1;
extern char empfbuffer2[];
extern char telegrnummer2;
extern char empfbuffer3[];
extern char telegrnummer3;
extern char addr_seg;
extern int addr_off;
extern char addr_val;
extern char addr_val_new;
extern char addr_ch;
extern char msgbuffer[];
extern char bfbusmode;
extern char stern;
extern char modus;
extern char ausgeben;
extern char volume;
extern char squelch;
extern char tunestep;
extern char i2c_control;
extern char i2c_richtung;
extern char i2c_buffer_pt;
extern char i2c_buffer_in[];
extern char i2c_buffer_out[];
extern char i2c_buffer_anz;
extern char ad_wandler_ctrlbyte;
extern char temperatur;

extern char io_e002;
extern char io_e003;
extern char pt;

/*************************************************************************
 * PHN
 *
 * wandelt val in ASCII-String in Buffer um (als Hex-Zahl)
 *
 * global: stellen - Anzahl auszugebener Stellen
 *         buf1cc7 - Ausgabebuffer
 *
 * in:   pos - Startposition [1..16]
 *       val - Wert
 * out:  -
 * ret:  -
 *
 */
void phn(unsigned char pos, int val)
{
    int ax;

    if (stellen == 3)
        buf1cc7h[pos - 1] = '0';

    if (stellen >= 2)
        {
        ax = (val >> 4) + '0';

        if (ax > '9')
            ax += 7;

        buf1cc7h[pos] = ax;
        }
    else
        buf1cc7h[pos] = ' ';

    ax = (val & 0x0f) + '0';

    if (ax > '9')
        ax += 7;

    buf1cc7h[pos+1] = ax;
}


/*************************************************************************
 * PHN2
 *
 * schreibt eine Hex-Zahl an die gewuenschte Position im Ausgabebuffer
 *
 * in:  pos - Position im Ausgabebuffer [1..16]
 *      val - zu schreibender Wert
 * out: -
 * ret: -
 *
 */
void phn2(unsigned char pos, int val)
{
    char *hex = "0123456789abcdef";

    if (stellen == 3)
        buf1cc7h[pos - 1] = '0';

    if (stellen >= 2)
        buf1cc7h[pos] = hex[(val & 0xf0) >> 4];
    else
        buf1cc7h[pos] = ' ';

    buf1cc7h[pos+1] = hex[val & 0x0f];
}

/*************************************************************************
 * LOESCHE_AUSBUFFER
 *
 * loescht den Ausgabebuffer
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void loesche_ausbuffer(void)
{
    int i;

    for (i = 1; i <= 0x10; i++)
        buf1cc7h[i] = ' ';
}

/*************************************************************************
 * FUELLE_TXBUFFER
 *
 * fuellt den Sendebuffer
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void fuelle_txbuffer(void)
{
    char *header = "\x00\x1a\x15\x02";
    char *s;
    int i;
    int p;

    s = txbuffer;

    warte_auf_sender_fertig();

    /* Header */

    for (i=0; i < 4; i++)
        {
        *s = header[i];
        s++;
        }

    /* Ausgabebuffer */

    for (i=0; i < 16; i++)
        {
        *s = buf1cc7h[i+1];
        s++;
        }

    /* Pruefsumme berechnen */

    p = 0;
    for (i=0; i < (txbuffer[2] - 1); i++)
        p ^= txbuffer[i];

    *s = p;
}

/*************************************************************************
 * GET_TASTE
 *
 * holt den Tastencode aus der Nachricht vom Bedienteil
 *
 * in:  nr - Nummer des zu verwendenden Empfangsbuffers
 * out: -
 * ret: Tastencode
 *
 */
int get_taste(int nr)
{
    int ret;

    switch (nr)
        {
        case 1:
            ret = empfbuffer1[5];
            break;
        case 2:
            ret = empfbuffer2[5];
            break;
        case 3:
            ret = empfbuffer3[5];
            break;
        default:
            ret = 0;
            break;
        }

    return(ret);
}

/*************************************************************************
 * SHOWCURSOR
 *
 * zeigt an entsprechender Stelle den Cursor an
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void showcursor(void)
{
    if ((crs_pos > 0) && (crs_pos <= 16))
        {
        if (crs_buf == 0)
            {
            crs_buf = 1;
            buf1cc7h[crs_pos] = 0x7f;
            }
        else
            crs_buf = 0;
        }
}

/*************************************************************************
 * BEARBEITE_ADDR
 *
 * laesst Adresse und deren Inhalt bearbeiten
 *
 * in:  taste - Tastencode
 * out: -
 * ret: -
 *
 */
void bearbeite_addr(int taste)
{
    char t;
    signed char crs_weiter;

    if (crs_pos == 0)
        {
        if (taste == C_TASTE)
            crs_pos = 1;
        }
    else
        {
        switch (taste)
            {
            case C_TASTE:    /* c */
                crs_pos = 0;
                addr_ch = 0;
                break;
            case HOCH_TASTE:    /* hoch */
                if (crs_pos > 8)
                    crs_pos -= 8;
                break;
            case RUNTER_TASTE:    /* runter */
                if (crs_pos < 9)
                    crs_pos += 8;
                break;
            case LINKS_TASTE:    /* links */
                if (crs_pos == 1)
                    crs_pos = 8;
                else if (crs_pos == 9)
                    crs_pos = 16;
                else
                    crs_pos--;
                break;
            case RECHTS_TASTE:    /* rechts */
                if (crs_pos == 8)
                    crs_pos = 1;
                else if (crs_pos == 16)
                    crs_pos = 9;
                else
                    crs_pos++;
                break;
            case RAUTE_TASTE:
                if ((crs_pos >= 7) && (addr_ch == 1))
                    {
                    poke();
                    addr_ch = 0;
                    }
                break;
                /*
            case RAUTE_TASTE:
                if ((crs_pos >= 15) && (addr_ch == 1))
                    {
                    if ((addr_seg == 0) && ((addr_off & 0xf000) == 0xd000))
                        addr_val = 0;
                    else
                        peek();
                    addr_val_new = addr_val;
                    addr_ch = 0;
                    }
                break;
                */
            case 0x30:
            case 0x31:
            case 0x32:
            case 0x33:
            case 0x34:
            case 0x35:
            case 0x36:
            case 0x37:
            case 0x38:
            case 0x39:
                crs_weiter = 0;

                /* Adresse */
                if (crs_pos == 1)
                    addr_seg = taste - '0';
                else if (crs_pos == 2)
                    addr_off = (addr_off & 0x0fff) | ((taste - '0') << 12);
                else if (crs_pos == 3)
                    addr_off = (addr_off & 0xf0ff) | ((taste - '0') << 8);
                else if (crs_pos == 4)
                    addr_off = (addr_off & 0xff0f) | ((taste - '0') << 4);
                else if (crs_pos == 5)
                    addr_off = (addr_off & 0xfff0) | (taste - '0');

                if ((crs_pos >= 1) && (crs_pos <= 5))
                    {
                    addr_ch = 0;
                    if (crs_pos < 5)
                        crs_weiter = 1;
                    else
                        crs_weiter = 2;
                    }

                /* Wert */
                if (crs_pos == 7)
                    {
                    addr_val_new = (addr_val_new & 0x0f) | ((taste - '0') << 4);
                    crs_weiter++;
                    addr_ch = 1;
                    }
                else if (crs_pos == 8)
                    {
                    addr_val_new = (addr_val_new & 0xf0) | (taste - '0');
                    addr_ch = 1;
                    }

                crs_pos += crs_weiter;
                break;
            case PUNKT_TASTE:  /* ein Punkt, Wert erhhen */
                /* Adresse */
                if (crs_pos == 1)
                    {
                    if (addr_seg < 0xf)
                        addr_seg++;
                    else
                        addr_seg = 0;
                    }
                else if (crs_pos == 2)
                    {
                    t = (addr_off & 0xf000) >> 12;
                    addr_off &= 0x0fff;

                    if (t < 0xf)
                        {
                        t++;
                        addr_off |= (t << 12);
                        }
                    }
                else if (crs_pos == 3)
                    {
                    t = (addr_off & 0x0f00) >> 8;
                    addr_off &= 0xf0ff;

                    if (t < 0xf)
                        {
                        t++;
                        addr_off |= (t << 8);
                        }
                    }
                else if (crs_pos == 4)
                    {
                    t = (addr_off & 0x00f0) >> 4;
                    addr_off &= 0xff0f;

                    if (t < 0xf)
                        {
                        t++;
                        addr_off |= (t << 4);
                        }
                    }
                else if (crs_pos == 5)
                    {
                    t = addr_off & 0x000f;
                    addr_off &= 0xfff0;

                    if (t < 0xf)
                        {
                        t++;
                        addr_off |= t ;
                        }
                    }

                if ((crs_pos >= 1) && (crs_pos <= 5))
                    addr_ch = 0;

                /* ############################# */

                if (crs_pos == 7)
                    {
                    t = (addr_val_new & 0xf0) >> 4;
                    addr_val_new &= 0x0f;

                    if (t < 0xf)
                        {
                        t++;
                        addr_val_new |= (t << 4);
                        }
                    addr_ch = 1;
                    }
                else if (crs_pos == 8)
                    {
                    t = addr_val_new & 0x0f;
                    addr_val_new &= 0xf0;

                    if (t < 0xf)
                        {
                        t++;
                        addr_val_new |= t ;
                        }
                    addr_ch = 1;
                    }

                break;

            case DPLPUNKT_TASTE:  /* Doppelpunkt, Wert verringern */
                /* Adresse */
                if (crs_pos == 1)
                    {
                    if (addr_seg > 0)
                        addr_seg--;
                    else
                        addr_seg = 0xf;
                    addr_ch = 0;
                    }
                else if (crs_pos == 2)
                    {
                    t = (addr_off & 0xf000) >> 12;
                    addr_off &= 0x0fff;

                    if (t > 0)
                        t--;
                    else
                        t = 0xf;

                    addr_off |= (t << 12);
                    addr_ch = 0;
                    }
                else if (crs_pos == 3)
                    {
                    t = (addr_off & 0x0f00) >> 8;
                    addr_off &= 0xf0ff;

                    if (t > 0)
                        t--;
                    else
                        t = 0xf;

                    addr_off |= (t << 8);
                    addr_ch = 0;
                    }
                else if (crs_pos == 4)
                    {
                    t = (addr_off & 0x00f0) >> 4;
                    addr_off &= 0xff0f;

                    if (t > 0)
                        t--;
                    else
                        t = 0xf;

                    addr_off |= (t << 4);
                    addr_ch = 0;
                    }
                else if (crs_pos == 5)
                    {
                    t = addr_off & 0x000f;
                    addr_off &= 0xfff0;

                    if (t > 0)
                        t--;
                    else
                        t = 0xf;

                    addr_off |= t ;
                    addr_ch = 0;
                    }

                /* ############################# */

                if (crs_pos == 7)
                    {
                    t = (addr_val_new & 0xf0) >> 4;
                    addr_val_new &= 0x0f;

                    if (t > 0)
                        t--;
                    else
                        t = 0xf;

                    addr_val_new |= (t << 4);
                    addr_ch = 1;
                    }
                else if (crs_pos == 8)
                    {
                    t = addr_val_new & 0x0f;
                    addr_val_new &= 0xf0;

                    if (t > 0)
                        t--;
                    else
                        t = 0xf;

                    addr_val_new |= t;
                    addr_ch = 1;
                    }

                break;
            }
        }

    if (taste == HOERER_TASTE)
        sendmsg();
}

/*************************************************************************
 * BEARBEITE_ZEIT
 *
 * laesst Zeit und Datum bearbeiten
 *
 * in:  taste - Tastencode
 * out: -
 * ret: -
 *
 */
void bearbeite_zeit(int taste)
{
    if (crs_pos == 0)
        {
        if (taste == C_TASTE)
            crs_pos = 1;
        }
    else
        {
        switch (taste)
            {
            case C_TASTE:    /* c */
                crs_pos = 0;
                break;
            case HOCH_TASTE:    /* hoch */
                if (crs_pos > 8)
                    crs_pos -= 8;
                break;
            case RUNTER_TASTE:    /* runter */
                if (crs_pos < 9)
                    crs_pos += 8;
                break;
            case LINKS_TASTE:    /* links */
                if (crs_pos == 1)
                    crs_pos = 8;
                else if (crs_pos == 9)
                    crs_pos = 16;
                else
                    crs_pos--;
                break;
            case RECHTS_TASTE:    /* rechts */
                if (crs_pos == 8)
                    crs_pos = 1;
                else if (crs_pos == 16)
                    crs_pos = 9;
                else
                    crs_pos++;
                break;
            case 0x30:
            case 0x31:
            case 0x32:
            case 0x33:
            case 0x34:
            case 0x35:
            case 0x36:
            case 0x37:
            case 0x38:
            case 0x39:

                /* Uhrzeit */
                if ((crs_pos == 1) && (taste <= 0x32))
                    {
                    hour = (hour % 10) + ((taste - '0') * 10);
                    crs_pos = 2;
                    }
                else if (crs_pos == 2)
                    {
                    hour = (hour / 10) * 10 + (taste - '0');
                    crs_pos = 4;
                    }
                else if ((crs_pos == 4) && (taste <= 0x35))
                    {
                    min = (min % 10) + ((taste - '0') * 10);
                    crs_pos = 5;
                    }
                else if (crs_pos == 5)
                    {
                    min = (min / 10) * 10 + (taste - '0');
                    crs_pos = 7;
                    }
                else if ((crs_pos == 7) && (taste <= 0x35))
                    {
                    sec = (sec % 10) + ((taste - '0') * 10);
                    crs_pos = 8;
                    }
                else if (crs_pos == 8)
                    {
                    sec = (sec / 10) * 10 + (taste - '0');
                    crs_pos = 1;
                    }

                /* Datum */

                else if ((crs_pos == 9) && (taste <= 0x33))
                    {
                    day = (day % 10) + ((taste - '0') * 10);
                    crs_pos = 10;
                    }
                else if (crs_pos == 10)
                    {
                    day = (day / 10) * 10 + (taste - '0');
                    crs_pos = 12;
                    }
                else if ((crs_pos == 12) && (taste <= 0x31))
                    {
                    mon = (mon % 10) + ((taste - '0') * 10);
                    crs_pos = 13;
                    }
                else if (crs_pos == 13)
                    {
                    mon = (mon / 10) * 10 + (taste - '0');
                    crs_pos = 15;
                    }
                else if (crs_pos == 15)
                    {
                    year = (year % 10) + ((taste - '0') * 10);
                    crs_pos = 16;
                    }
                else if (crs_pos == 16)
                    {
                    year = (year / 10) * 10 + (taste - '0');
                    crs_pos = 9;
                    }
                break;
            }
        }

}

/*************************************************************************
 * BIN2BCD
 *
 * wandelt binaer in BCD um
 *
 * in:  val - Binaerwert
 * out: -
 * ret: BCD-Wert
 *
 */
int bin2bcd(int val)
{
    char hi, lo;

    val &= 0x00ff;

    lo = val % 10;
    val /= 10;
    hi = val % 10;

    return((hi << 4) | lo);
}

/*************************************************************************
 * SHOWTIME
 *
 * gibt die Zeit und das Datum aus
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void showtime(void)
{
    stellen = 2;

    phn2(1, bin2bcd(hour));
    buf1cc7h[3] = ':';
    phn2(4, bin2bcd(min));
    buf1cc7h[6] = ':';
    phn2(7, bin2bcd(sec));

    phn2(9, bin2bcd(day));
    buf1cc7h[11] = '.';
    phn2(12, bin2bcd(mon));
    buf1cc7h[14] = '.';
    phn2(15, bin2bcd(year));
}

/*************************************************************************
 * SHOWTEMP
 *
 * gibt die Temperatur aus
 * Temperaturtabelle aus dem Originalprogramm
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void showtemp(void)
{
    char t;
    char *ttab = "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        /* 0d */ "\x14\x14\x14\x14\x14"
        /* 12 */ "\x15\x15\x15\x15"
        /* 16 */ "\x16\x16\x16\x16\x16"
        /* 1b */ "\x17\x17\x17\x17"
        /* 1f */ "\x18\x18\x18\x18"
        /* 23 */ "\x19\x19\x19\x19"
        /* 27 */ "\x1a\x1a\x1a\x1a"
        /* 2b */ "\x1b\x1b\x1b"
        /* 2e */ "\x1c\x1c\x1c\x1c"
        /* 32 */ "\x1d\x1d\x1d"
        /* 35 */ "\x1e\x1e\x1e\x1e\x1e"
        /* 3a */ "\x1f\x1f\x1f\x1f\x1f"
        /* 3f */ "\x20\x20\x20\x20"
        /* 43 */ "\x21\x21\x21\x21\x21"
        /* 48 */ "\x22\x22\x22\x22"
        /* 4c */ "\x23\x23\x23\x23\x23"
        /* 51 */ "\x24\x24\x24\x24\x24"
        /* 56 */ "\x25\x25\x25\x25"
        /* 5a */ "\x26\x26\x26\x26\x26"
        /* 5f */ "\x27\x27\x27\x27"
        /* 63 */ "\x28\x28\x28\x28\x28"
        /* 68 */ "\x29\x29\x29\x29"
        /* 6c */ "\x2a\x2a\x2a\x2a"
        /* 70 */ "\x2b\x2b\x2b\x2b"
        /* 74 */ "\x2c\x2c\x2c\x2c"
        /* 78 */ "\x2d\x2d\x2d\x2d\x2d"
        /* 7d */ "\x2e\x2e\x2e\x2e"
        /* 81 */ "\x2f\x2f\x2f\x2f"
        /* 85 */ "\x30\x30\x30\x30"
        /* 89 */ "\x31\x31\x31\x31"
        /* 8d */ "\x32\x32\x32\x32\x32"
        /* 92 */ "\x33\x33\x33\x33"
        /* 96 */ "\x34\x34\x34\x34"
        /* 9a */ "\x35\x35\x35\x35"
        /* 9e */ "\x36\x36\x36\x36"
        /* a2 */ "\x37\x37\x37\x37\x37"
        /* a7 */ "\x38\x38\x38\x38"
        /* ab */ "\x39\x39\x39\x39"
        /* af */ "\x3a\x3a\x3a\x3a"
        /* b3 */ "\x3b\x3b\x3b\x3b"
        /* b7 */ "\x3c\x3c\x3c\x3c\x3c"
        /* bc */ "\x3d\x3d\x3d\x3d"
        /* c0 */ "\x3e\x3e\x3e\x3e"
        /* c4 */ "\x3f\x3f\x3f\x3f"
        /* c8 */ "\x40\x40\x40\x40"
        /* cc */ "\x41\x41\x41\x41"
        /* d0 */ "\x42\x42\x42\x42"
        /* d4 */ "\x43\x43\x43\x43"
        /* d8 */ "\x44\x44\x44\x44"
        /* dc */ "\x45\x45\x45\x45"
        /* e0 */ "\x46";


    /* diese "Berechnung" ist aus dem Originalprogramm */

    if (temperatur < 0x0d)
        {
        buf1cc7h[10] = '<';
        t = 20;
        }
    else if (temperatur > 0xe0)
        {
        buf1cc7h[10] = '>';
        t = 70;
        }
    else
        {
        buf1cc7h[10] = '+';
        t = ttab[temperatur];
        }

    stellen = 2;

    /* Testausgabe */
    phn2(7, bin2bcd(temperatur));

    phn2(11, bin2bcd(t));
    buf1cc7h[14] = 'C';
}

/*************************************************************************
 * SHOWADDR
 *
 * gibt die aktuelle Adresse und deren Inhalt aus
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void showaddr(void)
{
    char *hex = "0123456789abcdef";
    char temp;

    stellen = 2;
    buf1cc7h[1] = hex[addr_seg & 0x0f];
    phn2(2, addr_off >> 8);
    phn2(4, addr_off % 256);

    if (addr_ch == 0)
        {
        if ((addr_seg == 0) && ((addr_off & 0xf000) == 0xd000))
            /* lesen auf dieser Adresse fuehrt zu schreiben! */
            addr_val = 0;
        else
            peek();
        addr_val_new = addr_val;

        temp = addr_val;
        phn2(7, addr_val);
        }
    else
        {
        buf1cc7h[6] = stern;
        phn2(7, addr_val_new);
        temp = addr_val_new;
        }

    stellen = 1;
    phn2(9, bin2bcd(temp / 100));
    stellen = 2;
    phn2(11, bin2bcd(temp % 100));

    buf1cc7h[14] = '\'';
    if ((temp >= 0x20) && (temp <= 0x91))
        buf1cc7h[15] = temp;
    else
        buf1cc7h[15] = '.';
    buf1cc7h[16] = '\'';

}

/*************************************************************************
 * LESE_AD
 *
 * in:  ch - AD-Wandler-Kanal [0..3]
 * out: -
 * ret: -
 *
 */
void lese_ad(char ch)
{
    lese_ad_vorb(ch);

    while ((i2c_control & 8) != 0)
        {
        i2c_ausgabe();
        }
}

/*************************************************************************
 * I2C_AUSGABE
 *
 * noch nicht fertig
 *
 * in:
 * out:
 * ret:
 *
 */
void i2c_ausgabe_c(void)
{
    if ((io_e003 & 8) != 0)
        return;

    /* AUSGABE_1 */
    if ((i2c_richtung & 1) == 0)
        {
        if (i2c_buffer_pt != 0)
            {
            i2c_buffer_in[i2c_buffer_pt - 1] = io_e002;
            }

        /* -> */ /* AUSGABE_5 */
        }
    else
        {
        /* AUSGABE_2 */

        if (io_e003 & 4 == 0)
            {
            }
        }
}

/*************************************************************************
 * I2C_AUSGABE_VORB
 *
 * in:  anz - Anzahl der Bytes im Buffer
 * out: -
 * ret: -
 *
 */
void i2c_ausgabe_vorb(int anz)
{
    if (anz != 0)
        {
        i2c_buffer_anz = anz >> 1;
        i2c_control |= 8;
        i2c_buffer_pt = 0;
        }
}

/*************************************************************************
 * LESE_AD_VORB
 *
 * bereitet alles fuer die I2C-Uebertragung zum Lesen des AD-Wandlers vor
 *
 * in:  ch - AD-Wandler-Kanal [0..3]
 * out: -
 * ret: -
 *
 */
void lese_ad_vorb(char ch)
{
    int anz;

    i2c_richtung = 0xff;

    if ((ad_wandler_ctrlbyte & ch) != ch)
        {
        ad_wandler_ctrlbyte = (ch | 0x40);
        i2c_buffer_out[0] = 0x90;
        i2c_buffer_out[1] = 0x05;
        i2c_buffer_out[2] = ad_wandler_ctrlbyte;
        i2c_buffer_out[3] = 0x06;
        i2c_buffer_out[4] = 0x91;
        i2c_buffer_out[5] = 0x05;
        i2c_buffer_out[6] = 0xff;
        i2c_buffer_out[7] = 0x00;
        i2c_buffer_out[8] = 0xff;
        i2c_buffer_out[9] = 0x06;
        anz = 10;
        }
    else
        {
        i2c_buffer_out[0] = 0x91;
        i2c_buffer_out[1] = 0x05;
        i2c_buffer_out[2] = 0xff;
        i2c_buffer_out[3] = 0x00;
        i2c_buffer_out[4] = 0xff;
        i2c_buffer_out[5] = 0x06;
        anz = 6;
        }

    i2c_ausgabe_vorb(anz);
}

/*************************************************************************
 * SENDMSG
 *
 * sendet die aktuelle Nachricht im msgbuffer
 * Pruefsumme wird berechnet
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void sendmsg(void)
{
    int i;
    int p;

    /* falsche Adresse oder zu kurz/lang ? */

    if ((msgbuffer[0] != 0x00) ||
        (msgbuffer[1] != 0x1a) ||
        (msgbuffer[2] < 4)     ||
        (msgbuffer[2] > 0x1c))
        return;

    /* warten, bis der vorige Sendevorgang fertig ist */
    warte_auf_sender_fertig();

    p = 0;
    for (i = 0; i < (msgbuffer[2] - 1); i++)
        {
        txbuffer[i] = msgbuffer[i];
        p ^= msgbuffer[i];
        }

    msgbuffer[i] = p;
    txbuffer[i] = p;

    sende_zum_bfbus();
}

/*************************************************************************
 * WARTE_AUF_SENDER_FERTIG
 *
 * wartet auf Abschluss des vorigen Sendevorgangs
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void warte_auf_sender_fertig(void)
{
    while ((bfbusmode < 2) || (bfbusmode > 4))
        {
        ;
        }
}

/*************************************************************************
 * BESTAETIGUNG
 *
 * schickt eine Bestaetigung zum Bedienteil
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void bestaetigung(void)
{
    warte_auf_sender_fertig();

    txbuffer[0] = 0x00;
    txbuffer[1] = 0x1a;
    txbuffer[2] = 0x05;
    txbuffer[3] = 0x10;
    txbuffer[4] = 0x0f;

    sende_zum_bfbus();
}

/*************************************************************************
 * PRINT
 *
 * schreibt Text in Buffer fuer Bedienteil
 * fuellt mit Leerzeichen auf, wenn kuerzer als 16 Zeichen
 *
 * in:  text - auszugebender String
 * out: -
 * ret: -
 *
 */
void print(char *text)
{
    int i;

    for (i=0; i < 16 && text[i] != 0; i++)
        buf1cc7h[i+1] = text[i];

    for (; i < 16; i++)
        buf1cc7h[i+1] = ' ';

}

/*************************************************************************
 * GIB_MODUS_AUS
 *
 * Menueausgabe
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void gib_modus_aus(void)
{
    loesche_ausbuffer();

    switch (modus)
        {
        case M_VOLUME :
            print("Volume");
            phn2(9, volume);
            break;
        case M_SQUELCH :
            print("Squelch");
            phn2(9, squelch);
            break;
        case M_TUNESTEP :
            print("TuneStep");
            phn2(9, tunestep);
            break;
        case M_WRITEMEM :
            print("WriteMem");
            break;
        case M_FREQ :
            print("Freq:");
            break;
        case M_TXPOWER :
            print("TxPower");
            break;
        case M_RELSHIFT :
            print("RelShift");
            break;
        case M_LIGHT :
            print("Light");
            break;
        case M_MEMVFO :
            print("MemVFO");
            break;
        case M_UBATT :
            print("UBatt");
            showtemp();
            break;
        case M_CPRIGHT :
            print("20010303byDB1OTM");
            break;
        case M_MEMNAME :
            print("MemName");
            break;
        case M_TIME :
            showtime();
            showcursor();
            break;
        case M_MONITOR :
            showaddr();
            showcursor();
            break;
        case M_AUS :
            print("   aus");
            break;
        }

    fuelle_txbuffer();
    sende_zum_bfbus();
}

/*************************************************************************
 * TASTE_BEARBEITEN
 *
 * auf Tastendruck reagieren, modusabhaengig
 *
 * in:  taste - Tastencode
 * out: -
 * ret: -
 *
 */
void taste_bearbeiten(int taste)
{
    switch (modus)
        {
        case M_VOLUME :

            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                case RECHTS_TASTE :
                    if (volume < 0xff)
                        volume++;
                    ausgeben = 1;
                    break;
                case LINKS_TASTE :
                    if (volume > 0)
                        volume--;
                    ausgeben = 1;
                    break;
                }
            break;

        case M_SQUELCH :

            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                case RECHTS_TASTE :
                    if (squelch < 0xff)
                        squelch++;
                    ausgeben = 1;
                    break;
                case LINKS_TASTE :
                    if (squelch > 0)
                        squelch--;
                    ausgeben = 1;
                    break;
                }
            break;

        case M_TUNESTEP :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                case RECHTS_TASTE :
                    if (tunestep < 0xff)
                        tunestep++;
                    ausgeben = 1;
                    break;
                case LINKS_TASTE :
                    if (tunestep > 0)
                        tunestep--;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_WRITEMEM :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_FREQ :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_TXPOWER :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_RELSHIFT :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_LIGHT :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_MEMVFO :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_UBATT :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_CPRIGHT :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_MEMNAME :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                }
            break;
        case M_TIME :
            switch (taste)
                {
                case STERN_TASTE :
                    modus++;
                    ausgeben = 1;
                    break;
                default :
                    bearbeite_zeit(taste);
                    ausgeben = 1;
                    break;
                }
            break;
        case M_MONITOR :
            switch (taste)
                {
                case STERN_TASTE :
                    modus = M_VOLUME;
                    addr_ch = 0;
                    ausgeben = 1;
                    break;
                default :
                    bearbeite_addr(taste);
                    ausgeben = 1;
                    break;
                }
            break;
        default :
            modus = M_VOLUME;
            ausgeben = 1;
            break;
        }

}

/*************************************************************************
 * HAUPTSCHLEIFE
 *
 * wird nach Initialisierung aufgerufen und nur zum Ausschalten
 * verlassen
 *
 * in:  -
 * out: -
 * ret: -
 *
 */
void hauptschleife(void)
{
    int taste;

    ausgeben = 1;

    while(1)
        {
        /* evtl. veraenderliche Texte ausgeben */
        /* ----------------------------------- */

        if (ausgeben)
            {
            gib_modus_aus();
            ausgeben = 0;
            }

        /* sind Telegramme von aussen gekommen ? */
        /* ------------------------------------- */
        if (telegrnummer1 != 0)
            {
            bestaetigung();
            telegrnummer1 = 0;
            taste = get_taste(1);
            }
        if (telegrnummer2 != 0)
            {
            bestaetigung();
            telegrnummer2 = 0;
            }
        if (telegrnummer3 != 0)
            {
            bestaetigung();
            telegrnummer3 = 0;
            }

        if (taste > 0)
            {
            taste_bearbeiten(taste);
            taste = 0;
            }

        if (hsec != hsec_alt)
            {
            hsec_alt = hsec;

            if ((modus == M_TIME) || (modus == M_MONITOR))
                ausgeben = 1;

            /* alle 5 Sekunden AD-Wandler lesen */

            if ((hsec % 10) == 0)
                {
                lese_ad(1);

                if (i2c_buffer_out[0] != 0x90)
                    temperatur = i2c_buffer_in[2];
                else
                    temperatur = i2c_buffer_in[4];

                if (modus == M_UBATT)
                    ausgeben = 1;
                }
            }

        /* Aus-Taste ? */
        /* ----------- */

        if ((pt & 4) == 0)
            break;

        }

    /* Ausschalten */
    /* ----------- */

    modus = M_AUS;
    gib_modus_aus();
}

